<?php
/*
Plugin Name: BMRE Domain Monitor
Plugin URI: http://dwall.in/plugins/bmre_domain_monitor
Description: Keeps track on your domains expiration and warns you before it is about to expire
Version: 0.3
Author: Dwall.in - David V. Wallin
Author URI: http://dwall.in
License: GPL2
*/
/*  Copyright 2012 David V. Wallin  (email : david@dwall.in)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

include('lib/whois.main.php');
$whois = new Whois();
global $bmre_domain_monitor_db_version;
$bmre_domain_monitor_db_version = "1.0";

function bmre_domain_monitor_install() {
   global $wpdb;
   global $bmre_domain_monitor_db_version;

   $table_name = $wpdb->prefix . "bmre_domain_monitor";
   $second_table_name = $wpdb->prefix . "bmre_domain_monitor_domain_checker";
      
   $sql = "CREATE TABLE $table_name (
  id int(11) unsigned NOT NULL AUTO_INCREMENT,
  domain varchar(128) NOT NULL,
  expires date NOT NULL,
  last_check date NOT NULL,
  notified int(1) NOT NULL DEFAULT '0',
  UNIQUE KEY id (id)
    );";
      
   $second_sql = "CREATE TABLE $second_table_name (
  id int(11) unsigned NOT NULL AUTO_INCREMENT,
  domain varchar(128) NOT NULL,
  last_check date NOT NULL,
  UNIQUE KEY id (id)
    );";


   require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
   dbDelta($sql);
   dbDelta($second_sql);
 
   add_option("bmre_domain_monitor_db_version", $bmre_domain_monitor_db_version);
}

register_activation_hook(__FILE__,'bmre_domain_monitor_install');

function bmre_domain_monitor()
{

}

add_action( 'admin_menu', 'bmre_domain_monitor_menu' );

function bmre_domain_monitor_menu() {
	add_options_page( 	'BMRE Domain Monitor', 
						'BMRE Domain Monitor', 
						'manage_options', 
						'bmre_domain_monitor', 
						'bmre_domain_monitor_options' );
}

function _make_safe($input=null)
{
	$replacement_array = array("*", "'", "\\", "\"", "{", "}", "(", ")", "[", "]", "=", "|", ":", "/", "https", "http", "www.");
	return str_replace($replacement_array, '', $input);
}

function bmre_domain_monitor_check()
{
	global $wpdb, $whois;
	$results = $wpdb->get_results("select * from " . $wpdb->prefix . "bmre_domain_monitor where last_check != '".date("Y-m-d")."'");

	if ( $results != 0 && $results != false && $results != null )
	{
			$domain_array = array();
			foreach ( $results as $item )
			{
				$domain_array[$item->id] = $item->domain;
			}
			foreach ( $domain_array as $domain_item_key => $domain_item_value )
			{
			   $result = "";
			   $result = $whois->Lookup($domain_item_value);
			   if ( array_key_exists('expires', $result['regrinfo']['domain']) )
			   {
			        $expires = $result['regrinfo']['domain']['expires'];
			   }else{
			       $expires = "Unknown";
			   }
			   $thirty_day_limit = date("Y-m-d", strtotime( '+30 days' ) );
			   if ( strtotime($expires) < strtotime($thirty_day_limit) )
			   {
			   		$notified_check = $wpdb->get_results(
			   				"select notified from " . $wpdb->prefix . "bmre_domain_monitor where id = ".$domain_item_key." limit 1"
			   			);
			   		if ( $notified_check[0]->notified < 2 )
			   		{
			   			wp_mail( 	get_bloginfo('admin_email'), 
					   				"Domain about to expire", 
					   				"The domain " . $domain_item_value . " is about to expire. The expiration-date is " . $expires . "." 
					   			);
			   			$wpdb->update($wpdb->prefix . "bmre_domain_monitor", array("notified"=>$notified_check[0]->notified + 1), array("id"=>$domain_item_key));
			   		}
		   		}else{
			   		$notified_check = $wpdb->get_results(
			   				"select notified from " . $wpdb->prefix . "bmre_domain_monitor where id = ".$domain_item_key." limit 1"
			   			);
			   		if ( $notified_check[0]->notified != 0 )
			   		{
			   			wp_mail( 	get_bloginfo('admin_email'), 
					   				"Domain about to expire", 
					   				"The domain " . $domain_item_value . " is about to expire. The expiration-date is " . $expires . "." 
					   			);
			   			$wpdb->update($wpdb->prefix . "bmre_domain_monitor", array("notified"=>0), array("id"=>$domain_item_key));
			   		}
			   }
				$wpdb->update($wpdb->prefix . "bmre_domain_monitor", array("expires"=>$expires, "last_check"=>date("Y-m-d")), array("id"=>$domain_item_key));
			}
	}
}
add_action( 'admin_head', 'bmre_domain_monitor_check' );

function bmre_domain_monitor_find_similar_domains($domainarray=null)
{
	if (	!is_array($domainarray) ||
			count($domainarray) == 0 ||
			$domainarray == null ||
			$domainarray == false )
	{
		return false;
	}
	global $wpdb, $whois;
	$returnation_array = array();
	$domain_endings_to_check = array('se', 'com', 'org', 'net', 'info', 'de', 'pl', 'dk', 'co.uk');
	foreach ( $domainarray as $array_item )
	{
		$exploded_domain = explode('.', $array_item->domain);
		$domain_endings_to_check[] = ltrim(str_replace($exploded_domain[0], '', $array_item->domain), '.');
		$domain_endings_to_check = array_unique($domain_endings_to_check);
		$thirty_day_limit = date("Y-m-d", strtotime( '-20 days' ) );
		foreach ( $domain_endings_to_check as $endint_check_item )
		{
	   		$domain_check = $wpdb->get_results(
	   				"select * from " . $wpdb->prefix . "bmre_domain_monitor_domain_checker 
	   				where domain = '".$exploded_domain[0] . "." . $endint_check_item."' limit 1"
	   			);
	   		if ( strtotime($domain_check[0]->last_check) < strtotime($thirty_day_limit) )
	   		{
				$domaininfo = $whois->Lookup($exploded_domain[0] . "." . $endint_check_item);
				foreach ( $domaininfo['rawdata'] as $rawrow_key => $rawrow_value )
				{
					if (	preg_match("/No match/i", $rawrow_value) ||
							preg_match("/not found/i", $rawrow_value) ||
							preg_match("/No information available/i", $rawrow_value) ||
							preg_match("/Status: free/i", $rawrow_value) ||
							preg_match("/No entries found/i", $rawrow_value) ||
							preg_match("/Status:	AVAILABLE/i", $rawrow_value) )
					{
						if (	$domain_check != false &&
								$domain_check != 0 &&
								$domain_check != null &&
								strtotime($domain_check[0]->last_check) < strtotime($thirty_day_limit) )
						{
							$wpdb->update(
								$wpdb->prefix . "bmre_domain_monitor_domain_checker", 
								array("last_check"=>date("Y-m-d")), 
								array("id"=>$domain_check[0]->id));
						}elseif (	$domain_check == false ||
									$domain_check == 0 ||
									$domain_check == null )
							{
							$wpdb->insert(	
								$wpdb->prefix . "bmre_domain_monitor_domain_checker", 
								array(	"domain"=>$exploded_domain[0] . "." . $endint_check_item, 
										"last_check"=>date("Y-m-d"))
							);
						}
					}
				}
	   		}
		}
	}
	return $returnation_array;
}

function bmre_domain_monitor_options() {
	if ( !current_user_can( 'manage_options' ) )  {
		wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
	}
	echo '<div class="wrap">';
	echo('<h2>BMRE Domain Monitor</h2>');
	global $wpdb;
	if ( isset($_GET['remove_domain']) && is_numeric($_GET['remove_domain']) )
	{
		$domain_id = _make_safe($_GET['remove_domain']);
		$domain = _make_safe($_GET['domain']);
		if ( !isset($_GET['confirm']) || $_GET['confirm'] != 1 )
		{
			echo('<p>You sure you wanna remove ' . $domain .'</p>');
			echo('<p>
					<a href="options-general.php?page=bmre_domain_monitor&remove_domain='.$domain_id.'&domain='.$domain.'&confirm=1">
						Yes
					</a> - 
					<strong>
						<a href="options-general.php?page=bmre_domain_monitor">
							No
						</a>
					</strong>
				</p>');
			exit;
		}
		$wpdb->query("	DELETE FROM " . $wpdb->prefix . "bmre_domain_monitor 
						WHERE id = ".$domain_id." 
						AND domain = '".$domain."'
						LIMIT 1");
		echo('<meta http-equiv="REFRESH" content="0;url='.get_bloginfo('url').'/wp-admin/options-general.php?page=bmre_domain_monitor">');
	}
	echo('<p>This plugin will check all added domains every day and notify the sites admin-mail when a domain expires within 30 days or less. It will notify two days in a row to be sure you get it.</p>');
	echo('<p>If you dont see a table below that is simply cause you haven\'t added a domain yet.</p>');
	$result = $wpdb->get_results("select * from " . $wpdb->prefix . "bmre_domain_monitor");
	if ( isset($_POST['submit']) && $_POST['domain_name'] != null && $_POST['domain_name'] != "" )
	{
		$domain_name = _make_safe($_POST['domain_name']);
		$check_for_domain_name = $wpdb->get_results("select * from " . $wpdb->prefix . "bmre_domain_monitor where domain = '".$domain_name."'");
		if ( $check_for_domain_name == 0 || $check_for_domain_name == null || $check_for_domain_name == false )
		{
			$wpdb->insert($wpdb->prefix . "bmre_domain_monitor", array("domain"=>$domain_name));

			echo('<meta http-equiv="REFRESH" content="0;url='.get_bloginfo('url').'/wp-admin/options-general.php?page=bmre_domain_monitor">');
		}
	}
	echo '<form id="add_domain" name="add_domain" action="" method="post">';
	echo '<label for="domain_name">Domain Name</label> ';
	echo '<input name="domain_name" id="domain_name" type="text" />';
	echo '<input name="submit" id="submit" type="submit" value="Add!" />';
	echo '</form>';
	echo '</div>';
	if ( $result != 0 && $result != false && $result != null )
	{
		#$similar_domain_names = bmre_domain_monitor_find_similar_domains($result);
		echo('<table style="width:100%;border:1px solid #ebebeb;">');
			echo('<thead>');
			echo('<tr style="text-align: left;">
				<th>Domain</th>
				<th>Expires</th>
				<th>Last Checked</th>
				<th>Action</th>
			</tr>');
			echo('</thead>');
			echo('<tbody>');
				foreach ( $result as $item )
				{
					$cssextra = "background:#d9fcc9;";
					$extrainfo = "";
				   	$thirty_day_limit = date("Y-m-d", strtotime( '+30 days' ) );
   					if ( strtotime($item->expires) < strtotime($thirty_day_limit) )
   					{
   						$cssextra = "background:#ffd1d4;";
   						$extrainfo = "<strong>(Expires in 30 or less days)</strong>";
   					}
   					if ( $item->expires == "Unknown" )
   					{
   						$cssextra = "background:#ffd1d4;";
   						$extrainfo = "<strong>(Cannot monitor this domain-type)</strong>";
   					}
   					if ( $item->expires == "0000-00-00" ) 
   					{
   						$cssextra = "background:#ffd1d4;";
   						$extrainfo = "<strong>(Cannot decide expiration-date)</strong>";
   					}					
					echo('<tr style="'.$cssextra.'">
							<td style="width:50%;">'.$item->domain.' '.$extrainfo.'</td>
							<td>'.$item->expires.'</td>
							<td>'.$item->last_check.'</td>
							<td>
								<a style="'.$cssextra.'" href="options-general.php?page=bmre_domain_monitor&remove_domain='.$item->id.'&domain='.$item->domain.'">
									Remove
								</a>
							</td>
						</tr>');
				}		
			echo('</tbody>');
		echo('</table>');
	}
}